const { deepStrictEqual, strictEqual } = require('assert')
const setup = require('./setup')
const formatError = (error) => {
  const { message, code, fieldName, context } = error.originalError

  return { message, code, fieldName, context }
}

describe('@constraint String', function () {
  const query = `mutation createBook($input: BookInput) {
    createBook(input: $input) {
      title
    }
  }`

  before(function () {
    this.typeDefs = `
        scalar DateTime

        type Query {
          books: [Book]
        }
        type Book {
          title: String
        }
        type Mutation {
          createBook(input: BookInput): Book
        }
        input BookInput {
          title: DateTime!
        }`

    this.request = setup(this.typeDefs)
  })

  it('should pass', async function () {
    const { body, statusCode } = await this.request
      .post('/graphql')
      .set('Accept', 'application/json')
      .send({
        query, variables: { input: { title: '2018-05-16 12:57:00Z' } }
      })

    strictEqual(statusCode, 200)
    deepStrictEqual(body, { data: { createBook: null } })
  })

  it('should fail', async function () {
    const { body, statusCode } = await this.request
      .post('/graphql')
      .set('Accept', 'application/json')
      .send({
        query, variables: { input: { title: 'a' } }
      })

    strictEqual(statusCode, 400)
    strictEqual(body.errors[0].message,
      'Variable "$input" got invalid value {"title":"a"}; Expected type ConstraintString at value.title; Must be a date in RFC 3339 format')
  })

  it('should throw custom error', async function () {
    const request = setup(this.typeDefs, formatError)
    const { body, statusCode } = await request
      .post('/graphql')
      .set('Accept', 'application/json')
      .send({ query, variables: { input: { title: 'a' } } })

    strictEqual(statusCode, 400)
    deepStrictEqual(body.errors[0], {
      message: 'Must be a date in RFC 3339 format',
      code: 'ERR_GRAPHQL_CONSTRAINT_VALIDATION',
      fieldName: 'title',
      context: [{ arg: 'format', value: 'date-time' }]
    })
  })
})
